@@ -1,13 +1,11 @@
-# -*- coding: utf-8 -*-
-from unittest.mock import patch, Mock
+from unittest.mock import Mock, patch
 
+from django.core.exceptions import ValidationError
 from django.db import connections
 from django.test import TestCase
 
 from explorer.app_settings import EXPLORER_DEFAULT_CONNECTION as CONN
-from explorer.models import (
-    QueryLog, Query, QueryResult, ColumnSummary, ColumnHeader
-)
+from explorer.models import ColumnHeader, ColumnSummary, Query, QueryLog, QueryResult
 from explorer.tests.factories import SimpleQueryFactory
 
 
@@ -80,23 +78,25 @@ class TestQueryModel(TestCase):
         log = QueryLog.objects.first()
         self.assertEqual(log.duration, res.duration)
 
+    @patch('explorer.models.s3_url')
     @patch('explorer.models.get_s3_bucket')
-    def test_get_snapshots_sorts_snaps(self, mocked_conn):
-        conn = Mock()
-        conn.list = Mock()
+    def test_get_snapshots_sorts_snaps(self, mocked_get_s3_bucket, mocked_s3_url):
+        bucket = Mock()
+        bucket.objects.filter = Mock()
         k1 = Mock()
-        k1.generate_url.return_value = 'http://s3.com/foo'
+        k1.key = 'foo'
         k1.last_modified = 'b'
         k2 = Mock()
-        k2.generate_url.return_value = 'http://s3.com/bar'
+        k2.key = 'bar'
         k2.last_modified = 'a'
-        conn.list.return_value = [k1, k2]
-        mocked_conn.return_value = conn
+        bucket.objects.filter.return_value = [k1, k2]
+        mocked_get_s3_bucket.return_value = bucket
+        mocked_s3_url.return_value = 'http://s3.com/presigned_url'
         q = SimpleQueryFactory()
         snaps = q.snapshots
-        self.assertEqual(conn.list.call_count, 1)
-        self.assertEqual(snaps[0].url, 'http://s3.com/bar')
-        conn.list.assert_called_once_with(prefix=f'query-{q.id}/snap-')
+        self.assertEqual(bucket.objects.filter.call_count, 1)
+        self.assertEqual(snaps[0].url, 'http://s3.com/presigned_url')
+        bucket.objects.filter.assert_called_once_with(Prefix=f'query-{q.id}/snap-')
 
     def test_final_sql_uses_merged_params(self):
         q = SimpleQueryFactory(sql="select '$$foo:bar$$', '$$qux$$';")
@@ -104,6 +104,14 @@ class TestQueryModel(TestCase):
         expected = "select 'bar', 'mux';"
         self.assertEqual(q.final_sql(), expected)
 
+    def test_final_sql_fails_blacklist_with_bad_param(self):
+        q = SimpleQueryFactory(sql="$$command$$ from bar;")
+        q.params = {'command': 'delete'}
+        expected = "delete from bar;"
+        self.assertEqual(q.final_sql(), expected)
+        with self.assertRaises(ValidationError):
+            q.execute_query_only()
+
     def test_cant_query_with_unregistered_connection(self):
         from explorer.utils import InvalidExplorerConnectionException
         q = SimpleQueryFactory(sql="select '$$foo:bar$$', '$$qux$$';", connection='not_registered')
