@@ -1,3 +1,4 @@
+import importlib
 import json
 import time
 from unittest.mock import Mock, patch
@@ -5,19 +6,27 @@ from unittest.mock import Mock, patch
 from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.db import connections
+from django.forms.models import model_to_dict
+from django.shortcuts import redirect
 from django.test import TestCase
 from django.urls import reverse
-from django.forms.models import model_to_dict
 
-from explorer.app_settings import (
-    EXPLORER_DEFAULT_CONNECTION as CONN,
-    EXPLORER_TOKEN
-)
-from explorer.models import Query, QueryLog, MSG_FAILED_BLACKLIST
-from explorer.tests.factories import SimpleQueryFactory, QueryLogFactory
+from explorer import app_settings
+from explorer.app_settings import EXPLORER_DEFAULT_CONNECTION as CONN
+from explorer.app_settings import EXPLORER_TOKEN
+from explorer.models import MSG_FAILED_BLACKLIST, Query, QueryLog
+from explorer.tests.factories import QueryLogFactory, SimpleQueryFactory
 from explorer.utils import user_can_see_query
 
 
+def reload_app_settings():
+    """
+    Reload app settings, otherwise changes from testing context manager won't take effect
+    app_settings are loaded at time of import
+    """
+    importlib.reload(app_settings)
+
+
 class TestQueryListView(TestCase):
 
     def setUp(self):
@@ -83,6 +92,10 @@ class TestQueryCreateView(TestCase):
         self.assertContains(resp, "New Query")
 
 
+def custom_view(request):
+    return redirect('/custom/login')
+
+
 class TestQueryDetailView(TestCase):
     databases = ['default', 'alt']
 
@@ -164,6 +177,30 @@ class TestQueryDetailView(TestCase):
         self.assertTemplateUsed(resp, 'explorer/query.html')
         self.assertNotContains(resp, '6872')
 
+    def test_doesnt_render_results_if_params_and_no_autorun(self):
+        with self.settings(EXPLORER_AUTORUN_QUERY_WITH_PARAMS=False):
+            reload_app_settings()
+            query = SimpleQueryFactory(sql='select 6870+3 where 1=$$myparam:1$$;')
+            resp = self.client.get(
+                reverse(
+                    "query_detail", kwargs={'query_id': query.id}
+                )
+            )
+            self.assertTemplateUsed(resp, 'explorer/query.html')
+            self.assertNotContains(resp, '6873')
+
+    def test_does_render_results_if_params_and_autorun(self):
+        with self.settings(EXPLORER_AUTORUN_QUERY_WITH_PARAMS=True):
+            reload_app_settings()
+            query = SimpleQueryFactory(sql='select 6870+4 where 1=$$myparam:1$$;')
+            resp = self.client.get(
+                reverse(
+                    "query_detail", kwargs={'query_id': query.id}
+                )
+            )
+            self.assertTemplateUsed(resp, 'explorer/query.html')
+            self.assertContains(resp, '6874')
+
     def test_admin_required(self):
         self.client.logout()
         query = SimpleQueryFactory()
@@ -172,6 +209,18 @@ class TestQueryDetailView(TestCase):
         )
         self.assertTemplateUsed(resp, 'admin/login.html')
 
+    def test_admin_required_with_explorer_no_permission_setting(self):
+        self.client.logout()
+        query = SimpleQueryFactory()
+        with self.settings(EXPLORER_NO_PERMISSION_VIEW='explorer.tests.test_views.custom_view'):
+            resp = self.client.get(
+                reverse("query_detail", kwargs={'query_id': query.id})
+            )
+            self.assertRedirects(
+                resp, '/custom/login',
+                target_status_code=404
+            )
+
     def test_individual_view_permission(self):
         self.client.logout()
         user = User.objects.create_user('user1', 'user@user.com', 'pwd')
@@ -234,14 +283,14 @@ class TestQueryDetailView(TestCase):
     @patch('explorer.models.get_s3_bucket')
     def test_query_snapshot_renders(self, mocked_conn):
         conn = Mock()
-        conn.list = Mock()
+        conn.objects.filter = Mock()
         k1 = Mock()
         k1.generate_url.return_value = 'http://s3.com/foo'
         k1.last_modified = '2015-01-01'
         k2 = Mock()
         k2.generate_url.return_value = 'http://s3.com/bar'
         k2.last_modified = '2015-01-02'
-        conn.list.return_value = [k1, k2]
+        conn.objects.filter.return_value = [k1, k2]
         mocked_conn.return_value = conn
 
         query = SimpleQueryFactory(sql="select 1;", snapshot=True)
@@ -257,7 +306,7 @@ class TestQueryDetailView(TestCase):
         query = SimpleQueryFactory(sql="select 1;")
         resp = self.client.post(
             reverse("query_detail", kwargs={'query_id': query.id}),
-            data={'sql': "select 'delete';"}
+            data={'sql': "delete from auth_user;"}
         )
         end = len(conn.queries)
 
@@ -427,6 +476,16 @@ class TestQueryPlayground(TestCase):
         resp = self.client.get(reverse("explorer_playground"))
         self.assertTemplateUsed(resp, 'admin/login.html')
 
+    def test_admin_required_with_no_permission_view_setting(self):
+        self.client.logout()
+        with self.settings(EXPLORER_NO_PERMISSION_VIEW='explorer.tests.test_views.custom_view'):
+            resp = self.client.get(reverse("explorer_playground"))
+            self.assertRedirects(
+                resp,
+                '/custom/login',
+                target_status_code=404
+            )
+
     def test_loads_query_from_log(self):
         querylog = QueryLogFactory()
         resp = self.client.get(
@@ -439,7 +498,7 @@ class TestQueryPlayground(TestCase):
     def test_fails_blacklist(self):
         resp = self.client.post(
             reverse("explorer_playground"),
-            {'sql': "select 'delete'"}
+            {'sql': "delete from auth_user;"}
         )
         self.assertTemplateUsed(resp, 'explorer/play.html')
         self.assertContains(resp, MSG_FAILED_BLACKLIST % '')
@@ -537,7 +596,7 @@ class TestSQLDownloadViews(TestCase):
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['content-type'], 'text/csv')
-        self.assertEqual(response.content.decode('utf-8'), '1|2\r\n1|2\r\n')
+        self.assertEqual(response.content.decode('utf-8-sig'), '1|2\r\n1|2\r\n')
 
     def test_sql_download_csv_with_tab_delim(self):
         url = reverse("download_sql") + '?format=csv&delim=tab'
@@ -546,7 +605,7 @@ class TestSQLDownloadViews(TestCase):
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['content-type'], 'text/csv')
-        self.assertEqual(response.content.decode('utf-8'), '1\t2\r\n1\t2\r\n')
+        self.assertEqual(response.content.decode('utf-8-sig'), '1\t2\r\n1\t2\r\n')
 
     def test_sql_download_csv_with_bad_delim(self):
         url = reverse("download_sql") + '?format=csv&delim=foo'
@@ -555,7 +614,7 @@ class TestSQLDownloadViews(TestCase):
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['content-type'], 'text/csv')
-        self.assertEqual(response.content.decode('utf-8'), '1,2\r\n1,2\r\n')
+        self.assertEqual(response.content.decode('utf-8-sig'), '1,2\r\n1,2\r\n')
 
     def test_sql_download_json(self):
         url = reverse("download_sql") + '?format=json'
