@@ -1,20 +1,11 @@
 from django.urls import path, re_path
 
 from explorer.views import (
-    QueryView,
-    CreateQueryView,
-    PlayQueryView,
-    DeleteQueryView,
-    ListQueryView,
-    ListQueryLogView,
-    DownloadFromSqlView,
-    DownloadQueryView,
-    StreamQueryView,
-    EmailCsvQueryView,
-    SchemaView,
-    format_sql,
+    CreateQueryView, DeleteQueryView, DownloadFromSqlView, DownloadQueryView, EmailCsvQueryView, ListQueryLogView,
+    ListQueryView, PlayQueryView, QueryView, SchemaView, StreamQueryView, format_sql,
 )
 
+
 urlpatterns = [
     path(
         '<int:query_id>/', QueryView.as_view(), name='query_detail'
