@@ -1,27 +1,55 @@
-import functools
 import re
+from collections import deque
+from typing import Iterable, Tuple
 
+from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.contrib.auth.forms import AuthenticationForm
 from django.contrib.auth.views import LoginView
-from django.contrib.auth import REDIRECT_FIELD_NAME
+
+import boto3
+import sqlparse
 from sqlparse import format as sql_format
+from sqlparse.sql import Token, TokenList
+from sqlparse.tokens import Keyword
 
 from explorer import app_settings
 
+
 EXPLORER_PARAM_TOKEN = "$$"
 
 
-def passes_blacklist(sql):
-    clean = functools.reduce(
-        lambda sql, term: sql.upper().replace(term, ""),
-        [t.upper() for t in app_settings.EXPLORER_SQL_WHITELIST],
-        sql
-    )
+def passes_blacklist(sql: str) -> Tuple[bool, Iterable[str]]:
+    sql_strings = sqlparse.split(sql)
+    keyword_tokens = set()
+    for sql_string in sql_strings:
+        statements = sqlparse.parse(sql_string)
+        for statement in statements:
+            for token in walk_tokens(statement):
+                if not token.is_whitespace and not isinstance(token, TokenList):
+                    if token.ttype in Keyword:
+                        keyword_tokens.add(str(token.value).upper())
+
     fails = [
-        bl_word for bl_word in app_settings.EXPLORER_SQL_BLACKLIST
-        if bl_word in clean.upper()
+        bl_word
+        for bl_word in app_settings.EXPLORER_SQL_BLACKLIST
+        if bl_word.upper() in keyword_tokens
     ]
-    return not any(fails), fails
+
+    return not bool(fails), fails
+
+
+def walk_tokens(token: TokenList) -> Iterable[Token]:
+    """
+    Generator to walk all tokens in a Statement
+    https://stackoverflow.com/questions/54982118/parse-case-when-statements-with-sqlparse
+    :param token: TokenList
+    """
+    queue = deque([token])
+    while queue:
+        token = queue.popleft()
+        if isinstance(token, TokenList):
+            queue.extend(token)
+        yield token
 
 
 def _format_field(field):
@@ -159,20 +187,26 @@ def get_valid_connection(alias=None):
 
 
 def get_s3_bucket():
-    from boto.s3.connection import S3Connection
-
-    conn = S3Connection(app_settings.S3_ACCESS_KEY,
-                        app_settings.S3_SECRET_KEY)
-    return conn.get_bucket(app_settings.S3_BUCKET)
+    kwargs = {
+        'aws_access_key_id': app_settings.S3_ACCESS_KEY,
+        'aws_secret_access_key': app_settings.S3_SECRET_KEY,
+        'region_name': app_settings.S3_REGION
+    }
+    if app_settings.S3_ENDPOINT_URL:
+        kwargs['endpoint_url'] = app_settings.S3_ENDPOINT_URL
+    s3 = boto3.resource('s3', **kwargs)
+    return s3.Bucket(name=app_settings.S3_BUCKET)
 
 
 def s3_upload(key, data):
-    from boto.s3.key import Key
     bucket = get_s3_bucket()
-    k = Key(bucket)
-    k.key = key
-    k.set_contents_from_file(data, rewind=True)
-    k.set_acl('public-read')
-    k.set_metadata('Content-Type', 'text/csv')
-    return k.generate_url(expires_in=0, query_auth=False)
+    bucket.upload_fileobj(data, key, ExtraArgs={'ContentType': "text/csv"})
+    return s3_url(bucket, key)
 
+
+def s3_url(bucket, key):
+    url = bucket.meta.client.generate_presigned_url(
+        ClientMethod='get_object',
+        Params={'Bucket': app_settings.S3_BUCKET, 'Key': key},
+        ExpiresIn=app_settings.S3_LINK_EXPIRATION)
+    return url
