@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 from django.shortcuts import get_object_or_404
 from django.views.generic.base import View
 
@@ -21,7 +20,7 @@ class DownloadFromSqlView(PermissionRequiredMixin, View):
     permission_required = 'view_permission'
 
     def post(self, request, *args, **kwargs):
-        sql = request.POST.get('sql')
+        sql = request.POST.get('sql', '')
         connection = request.POST.get('connection', '')
         query = Query(sql=sql, connection=connection, title='')
         ql = query.log(request.user)
